#define _CRT_SECURE_NO_WARNINGS

#include "sslinstr.h"

static unsigned char PUSH_RDX[] = { 0x52 };
static unsigned char PUSH_R8[] = { 0x41, 0x50 };
static unsigned char PUSH_551[] = { 0x68, 0x51, 0x05, 0x00, 0x00 };
static unsigned char PUSH_552[] = { 0x68, 0x52, 0x05, 0x00, 0x00 };

static unsigned char NEWLINE[] = { 0x0D, 0x0A };

void SSLInstr::OnModuleLoaded(void* module, char* module_name) {
  if (IsInstrumentModule(module_name)) {
    DWORD read_offset = GetProcOffset((HMODULE)module, "SSL_read");
    DWORD write_offset = GetProcOffset((HMODULE)module, "SSL_write");
    if (!read_offset || !write_offset) {
      FATAL("Error getting SSL_read/write offsets");
    }
    ssl_read_addr = (size_t)module + read_offset;
    ssl_write_addr = (size_t)module + write_offset;

    FILE* fp = fopen("ssllog.txt", "wb");
    if (fp) {
      fclose(fp);
    }

    replace_buf_remote = (char *)RemoteAllocate(1024 * 1024, READWRITE);
  }

  TinyInst::OnModuleLoaded(module, module_name);
}

InstructionResult SSLInstr::InstrumentInstruction(ModuleInfo* module,
  Instruction& inst,
  size_t bb_address,
  size_t instruction_address)
{
  if (instruction_address == ssl_write_addr) {
    WriteCode(module, PUSH_R8, sizeof(PUSH_R8));
    assembler_->OffsetStack(module, -32);
    WriteCode(module, PUSH_552, sizeof(PUSH_552));
    write_breakpoint_addr = GetCurrentInstrumentedAddress(module);
    assembler_->Breakpoint(module);
  }

  if (instruction_address == ssl_read_addr) {
    WriteCode(module, PUSH_RDX, sizeof(PUSH_RDX));
    assembler_->OffsetStack(module, -32);
    WriteCode(module, PUSH_551, sizeof(PUSH_551));
  }

  return INST_NOTHANDLED;
}

bool SSLInstr::OnException(Exception* exception_record) {
  if ((exception_record->type == BREAKPOINT) &&
    ((size_t)exception_record->ip == write_breakpoint_addr))
  {
    size_t buf_address = GetRegister(RDX);
    size_t buf_size = GetRegister(R8);

    char* buf = (char *)malloc(buf_size + 1);

    RemoteRead((void*)buf_address, buf, buf_size);
    buf[buf_size] = 0;

    bool is_printable = true;
    for (size_t i = 0; i < buf_size; i++) {
      if (!isprint((unsigned char)buf[i])) {
        is_printable = false;
        break;
      }
    }

    if (is_printable) {
      printf("SSL_write: \n%s\n", buf);
    }

    if(is_printable || (buf[0] == '<')) {
      FILE* fp = fopen("ssllog.txt", "ab");
      if (fp) { 
        fwrite("SSL_write:", strlen("SSL_write:"), 1, fp);
        fwrite(NEWLINE, sizeof(NEWLINE), 1, fp);
        fwrite(buf, buf_size, 1, fp);
        fwrite(NEWLINE, sizeof(NEWLINE), 1, fp);
        fclose(fp);
      }

      size_t replace_size = Replace(buf, buf_size);
      if (replace_size) {
        SetRegister(RDX, (size_t)replace_buf_remote);
        SetRegister(R8, replace_size);
      }
    } else {
      // printf("SSL_write: [non printable]\n");
    }

    free(buf);

    return true;
  }

  if ((exception_record->type == ACCESS_VIOLATION) &&
    exception_record->maybe_execute_violation &&
    ((size_t)exception_record->access_address == 0x552))
  {
    size_t sp = GetRegister(RSP);
    sp += 32;
    uint64_t write_count;
    RemoteRead((void*)sp, &write_count, sizeof(write_count));
    sp += 8;
    uint64_t ret_addr;
    RemoteRead((void*)sp, &ret_addr, sizeof(ret_addr));
    sp += 8;
    SetRegister(RAX, write_count);
    SetRegister(RSP, sp);
    SetRegister(RIP, ret_addr);

    return true;
  }

  if ((exception_record->type == ACCESS_VIOLATION) &&
    exception_record->maybe_execute_violation &&
    ((size_t)exception_record->access_address == 0x551))
  {
    int ret = (int)GetRegister(RAX);
    size_t sp = GetRegister(RSP);
    sp += 32;
    if (ret > 0) {
      uint64_t buf_address;
      RemoteRead((void*)sp, &buf_address, sizeof(buf_address));
      size_t buf_size = ret;

      char* buf = (char*)malloc(buf_size + 1);

      RemoteRead((void*)buf_address, buf, buf_size);
      buf[buf_size] = 0;

      bool is_printable = true;
      for (size_t i = 0; i < buf_size; i++) {
        if (!isprint((unsigned char)buf[i])) {
          is_printable = false;
          break;
        }
      }

      if (is_printable) {
        printf("SSL_read: \n%s\n", buf);
      }

      if(is_printable || (buf[0] == '<')) {

        FILE* fp = fopen("ssllog.txt", "ab");
        if (fp) {
          fwrite("SSL_read:", strlen("SSL_read:"), 1, fp);
          fwrite(NEWLINE, sizeof(NEWLINE), 1, fp);
          fwrite(buf, buf_size, 1, fp);
          fwrite(NEWLINE, sizeof(NEWLINE), 1, fp);
          fclose(fp);
        }

      } else {
        // printf("SSL_read: [non printable]\n");
      }

      free(buf);
    }
    sp += 8;
    uint64_t ret_addr;
    RemoteRead((void*)sp, &ret_addr, sizeof(ret_addr));
    sp += 8;
    SetRegister(RSP, sp);
    SetRegister(RIP, ret_addr);

    return true;
  }

  return TinyInst::OnException(exception_record);
}

char* SSLInstr::ReadTextFile(char* filename) {
  FILE* fp = fopen(filename, "rb");
  if (!fp) return NULL;

  fseek(fp, 0, SEEK_END);
  size_t size = ftell(fp);
  fseek(fp, 0, SEEK_SET);

  char* buf = (char *)malloc(size + 1);
  fread(buf, 1, size, fp);
  buf[size] = 0;

  fclose(fp);

  return buf;
}

size_t SSLInstr::Replace(char* buf, size_t buf_size) {
  char* seek = ReadTextFile("search.txt");
  if (!seek) return 0;

  size_t seek_len = strlen(seek);
  if (!seek_len) {
    free(seek);
    return 0;
  }

  char* pos = strstr(buf, seek);
  if (!pos) {
    free(seek);
    return 0;
  }
  size_t index = (size_t)pos - (size_t)buf;

  char* replace = ReadTextFile("replace.txt");
  if (!replace) {
    free(seek);
    return 0;
  }

  size_t replace_len = strlen(replace);

  size_t replace_buf_size = buf_size + replace_len - seek_len;
  char* replace_buf = (char*)malloc(replace_buf_size);

  memcpy(replace_buf, buf, index);
  memcpy(replace_buf + index, replace, replace_len);
  memcpy(replace_buf + index + replace_len, buf + index + seek_len, buf_size - index - seek_len);

  RemoteWrite(replace_buf_remote, replace_buf, replace_buf_size);

  free(seek);
  free(replace);
  free(replace_buf);

  return (int)replace_buf_size;
}
